; OptionBox
; A window that prompts to choose one of several choices.
; Inputs: hl -> OptionBox structure
; Outputs: a = Index of chosen option
;          e = Index of chosen option
;
; OptionBox Structure:
; .db "Title",0                     ; Title
; .db num_lines                     ; Number of text lines
; .db "Text line 1",0               ; Text lines
; .db "Text line 2",0               ; ...
; .db "Text line 3",0               ; ...
; .db num_choices                   ; Number of choices
; .db "Choice 0",0                  ; Choices
; .db "Choice 1",0                  ; ...
; .db "Choice 2",0                  ; ...
;
; Comments:
;  Output value is zero for the leftmost choice, and incremented to the right.
;  This window is vertically two pixels (one up, one down) larger than a MessageBox.
;  Title text is centered, the rest is left aligned.
;  Each option has one space between it and the next.
;  You may have zero lines of text, but you are not allowed to have zero options.
;

guiOptionBox:
 ; Almost identical to the messagebox routine except location changes
 res textwrite,(iy+sgrflags)
 set plotLoc,(iy+plotFlags)
 res fracDrawLFont,(iy+fontFlags)
 push hl
 ld hl,13*256+22
 ld de,47*256+73
 B_CALL _DrawRectBorderC
 ld hl,14*256+23
 ld de,19*256+72
 B_CALL _FillRect
 pop hl
 set textinverse,(iy+textflags)
 ld de,13*256+47
 ld (pencol),de
 call centertext
 res textinverse,(iy+textflags)
 ld de,20*256+24
 call multilinetextL
 ld e,0
guiOptionBoxRedraw:
 ld d,0
 push hl
 ld hl,39*256+24
 ld (pencol),hl
 pop hl
 set texterasebelow,(iy+textflags)
 ld b,(hl)
 push hl
 push bc
 inc hl
guiOptionBoxDrawLoop:
 ld a,d
 cp e
 jr nz,guiOptionBoxNotInverse
 set textinverse,(iy+textflags)
guiOptionBoxNotInverse:
 B_CALL _VPutS
 res textinverse,(iy+textflags)
 push de
 push hl
 ld a,lspace
 B_CALL _VPutMap
 pop hl
 pop de
 inc d
 djnz guiOptionBoxDrawLoop
 pop bc
 pop hl
guiOptionBoxWaitkeyLoop:
 ei
 halt
 push hl
 B_CALL _getcsc
 pop hl
 cp sk2nd
 jr z,guiOptionBoxSelect
 cp skEnter
 jr z,guiOptionBoxSelect
 sub 2
 jr z,guiOptionBoxLeft
 dec a
 jr nz,guiOptionBoxWaitkeyLoop
guiOptionBoxRight:
 ld a,e
 inc a
 cp b
 jr z,guiOptionBoxWaitkeyLoop
 ld e,a
 jr guiOptionBoxRedraw
guiOptionBoxLeft:
 ld a,e
 or a
 jr z,guiOptionBoxWaitkeyLoop
 dec a
 ld e,a
 jr guiOptionBoxRedraw
guiOptionBoxSelect:
 ld a,e
 ret


; OptionBoxEX
; A window that prompts for one of several choices, with extra options.
; Inputs: hl -> OptionBoxEX structure
; Outputs: a = Option chosen
;
; OptionBoxEX structure:
; .db flags                            ; see below
; .db height,width,titleheight         ; height of entire box, width of entire box, height of dark region
; .db num_choices                      ; number of choices
; .db defaultchoice                    ; choice highlighted at the beginning (0=first)
; .db rowlength                        ; Amount to move if up/down is pressed
; .db num_titles                       ; number of lines of text in the title
; .db "Title Line 1",0                 ; the actual lines of text for the title
; .db "Title Line 2",0                 ; ...
; .db num_lines                        ; number of lines of text in the box (not including title)
; .db "Line 1",0                       ; the actual lines of text
; .db "Line 2",0                       ; ...
; .db "Line 3",0                       ; ...
; .db "Line 4",0                       ; ...
; .db c1Index                          ; Return value if this choice is picked
; .db c1col,c1row                      ; pencol/penrow to display choice at
; .db "Choice One",0                   ; Text for the choice
; .db c2Index,c2col,c2row              ; ...
; .db "Choice Two",0                   ; ...
; .db c3Index,c3col,c3row              ; ...
; .db "Choice Three",0                 ; ...
; [.dw callback]                       ; Custom text routine
;
; Flags:                               SET (nz)                         RESET (z)
;  Bit 0: OB_Buffer                    Copy to screen + buffer          Copy to screen only
;  Bit 1: OB_CenterTitle               Center title horizontally        Left align title in box
;  Bit 2: OB_CenterText                Center text horizontally         Left align text in box
;  Bit 3: OB_CustomText                Custom callback routine exists   No custom callback routine
;  Bit 4: OB_BigTitle                  Display title in large font      Display title in small font
;  Bit 5: OB_BigText                   Display text in large font       Display text in small font
;  Bit 6: OB_BigChoices                Display choices in large font    Display choices in small font
;  Bit 7: OB_UpDownOnly                See below                        Normal cursor operation
;
; Comments:
;  The box will be centered on the screen.
;  Rowlength should be zero unless the options are arranged in a gridlike pattern.
;  If OB_UpDownOnly is set, pressing left or right will do nothing, and pressing
;   up or down will move one choice backward or forward, regardless of rowlength.
;  TextEraseBelow is set, so don't put the choices too close together.
;  The customtext callback is called after the choices are redrawn. This occurs
;   every time the user changes highlighted selections. The current selection is
;   stored in register "e". You can use this for such things as popup
;   explanations of the choices. The flash page is not reset, so you cannot use
;   MirageOS calls during any callbacks (and you don't need the macro for gcalls)
;   You do not need to preserve any registers in the callback.
;

OB_Buffer         equ   0
OB_CenterTitle    equ   1
OB_CenterText     equ   2
OB_CustomText     equ   3
OB_BigTitle       equ   4
OB_BigText        equ   5
OB_BigChoices     equ   6
OB_UpDownOnly     equ   7

guiOptionBoxEX:
 push hl
 push hl
 pop ix
 set plotLoc,(iy+plotFlags)
 res textwrite,(iy+sgrflags)
 bit OB_Buffer,(ix)
 jr z,guiOptionBoxEXNoBuffer
 res plotLoc,(iy+plotFlags)
 set textwrite,(iy+sgrflags)
guiOptionBoxEXNoBuffer:
 ld a,(ix+1)      ;height
 sra a
 neg
 add a,31
 ld h,a
 add a,(ix+1)
 ld d,a
 ld a,(ix+2)      ;width
 sra a
 neg
 add a,47
 ld l,a
 add a,(ix+2)
 ld e,a
 push hl
 push de
 push ix
 B_CALL _DrawRectBorderC
 pop ix
 pop de
 pop hl
 ld a,(ix+3)
 add a,h
 ld d,a
 B_CALL _FillRect
 pop hl           ;get pointer again
 ld de,7
 add hl,de
 set textinverse,(iy+textflags)
 res fracDrawLFont,(iy+fontFlags)
 bit OB_BigTitle,(ix)
 jr z,guiOptionBoxEXTitleSmall
 set fracDrawLFont,(iy+fontFlags)
guiOptionBoxEXTitleSmall:
 bit OB_CenterTitle,(ix)
 jr z,guiOptionBoxEXTitleLeft
 ld a,(ix+1)
 sra a
 neg
 add a,31
 bit fracDrawLFont,(iy+fontFlags)
 jr z,guiOptionBoxEXTitleNoShift
 inc a
guiOptionBoxEXTitleNoShift:
 ld d,a
 ld e,47
 ld (pencol),de
 call multilinetextC
 jr guiOptionBoxEXTitleDone
guiOptionBoxEXTitleLeft:
 ld a,(ix+1)
 sra a
 neg
 add a,31
 bit fracDrawLFont,(iy+fontFlags)
 jr z,guiOptionBoxEXTitleNoShift2
 inc a
guiOptionBoxEXTitleNoShift2:
 ld d,a
 ld a,(ix+2)
 sra a
 neg
 add a,48
 ld e,a
 ld (pencol),de
 call multilinetextL
guiOptionBoxEXTitleDone:
 res textinverse,(iy+textflags)
 res fracDrawLFont,(iy+fontFlags)
 bit OB_BigText,(ix)
 jr z,guiOptionBoxEXTextSmall
 set fracDrawLFont,(iy+fontFlags)
guiOptionBoxEXTextSmall:
 bit OB_CenterText,(ix)
 jr z,guiOptionBoxEXTextLeft
 ld a,(ix+1)
 sra a
 neg
 add a,32
 bit fracDrawLFont,(iy+fontFlags)
 jr z,guiOptionBoxEXTextNoShift
 inc a
guiOptionBoxEXTextNoShift:
 add a,(ix+3)
 ld d,a
 ld e,47
 call multilinetextC
 jr guiOptionBoxEXTextDone
guiOptionBoxEXTextLeft:
 ld a,(ix+1)
 sra a
 neg
 add a,32
 bit fracDrawLFont,(iy+fontFlags)
 jr z,guiOptionBoxEXTextNoShift2
 inc a
guiOptionBoxEXTextNoShift2:
 add a,(ix+3)
 ld d,a
 ld a,(ix+2)
 sra a
 neg
 add a,49
 ld e,a
 call multilinetextL
guiOptionBoxEXTextDone:
 ld e,(ix+5)
guiOptionBoxEXRedraw:
 ld d,0
 push hl
 set texterasebelow,(iy+textflags)
 ld b,(ix+4)
 push bc
 bit OB_BigChoices,(ix)
 jr z,guiOptionBoxEXDrawLoop
 set fracdrawlfont,(iy+fontflags)
guiOptionBoxEXDrawLoop:
 inc hl
 push de
 push bc
 ld de,pencol
 ldi
 ldi
 pop bc
 pop de
 ld a,d
 cp e
 jr nz,guiOptionBoxEXNotInverse
 set textinverse,(iy+textflags)
guiOptionBoxEXNotInverse:
 B_CALL _VPutS
 res textinverse,(iy+textflags)
 inc d
 djnz guiOptionBoxEXDrawLoop
 bit OB_Buffer,(ix)
 jr z,guiOptionBoxEXNoBuffer2
 push de
 push hl
 B_CALL _grbufcpy
 pop hl
 pop de
guiOptionBoxEXNoBuffer2:
 ;do the callback here, hl->->routine
 bit OB_CustomText,(ix)
 jr z,guiOptionBoxEXNoCallback
 push de
 ld bc,guiOptionBoxEXCallbackRet
 push bc
 ld a,(hl)
 inc hl
 ld h,(hl)
 ld l,a
 jp (hl)
guiOptionBoxEXCallbackRet:
 pop de
guiOptionBoxEXNoCallback:
 pop bc
 pop hl
guiOptionBoxEXWaitkeyLoop:
 ei
 halt
 push hl
 B_CALL _getcsc
 pop hl
 cp sk2nd
 jr z,guiOptionBoxEXSelect
 cp skEnter
 jr z,guiOptionBoxEXSelect
 dec a
 jr z,guiOptionBoxEXDown
 dec a
 jr z,guiOptionBoxEXLeft
 dec a
 jr z,guiOptionBoxEXRight
 dec a
 jr nz,guiOptionBoxEXWaitkeyLoop
guiOptionBoxEXUp:
 bit OB_UpDownOnly,(ix)
 jr nz,guiOptionBoxEXLeft2
 ld a,e
 sub (ix+6)
 jr c,guiOptionBoxEXWaitkeyLoop
 ld e,a
 jr guiOptionBoxEXRedraw
guiOptionBoxEXDown:
 bit OB_UpDownOnly,(ix)
 jr nz,guiOptionBoxEXRight2
 ld a,e
 add a,(ix+6)
 cp b
 jr nc,guiOptionBoxEXWaitkeyLoop
 ld e,a
 jp guiOptionBoxEXRedraw
guiOptionBoxEXLeft:
 bit OB_UpDownOnly,(ix)
 jr nz,guiOptionBoxEXWaitkeyLoop
guiOptionBoxEXLeft2:
 ld a,e
 or a
 jr z,guiOptionBoxEXWaitkeyLoop
 dec a
 ld e,a
 jp guiOptionBoxEXRedraw
guiOptionBoxEXRight:
 bit OB_UpDownOnly,(ix)
 jr nz,guiOptionBoxEXWaitkeyLoop
guiOptionBoxEXRight2:
 ld a,e
 inc a
 cp b
 jr z,guiOptionBoxEXWaitkeyLoop
 ld e,a
 jp guiOptionBoxEXRedraw
guiOptionBoxEXSelect:
 ld a,e
 or a
 jr z,guiOptionBoxEXFirst
guiOptionBoxStrLoop:
 inc hl
 inc hl
 inc hl
 call nextstr
 dec a
 jr nz,guiOptionBoxStrLoop
guiOptionBoxEXFirst:
 ld a,(hl)
 ld e,a
 res texterasebelow,(iy+textflags)
 res fracdrawlfont,(iy+fontflags)
 res textwrite,(iy+sgrflags)
 res plotloc,(iy+plotflags)
 ret
